nextflow.enable.dsl=2

cogentds_script = "${params.cogent_root}" + '/lib/report/CogentDS_DataHandler_and_Reporter.R'
cogentds_reporter = "${params.cogent_root}" + '/lib/report/'
conda_env_path = "${params.cogent_root}" + '/config/env/cogentds_report.yaml'

process COGENTDS_REPORT {
    publishDir "${params.output_dir}", mode: 'copy'
    conda conda_env_path 

    input:
        path (analyze_stats)
        path (counts_rds)
        path (clonotype_counts)
        path (clonotype_summary)
        path (clonotype_metadata)
        path (fusion_results)
        val (umi_only)
        path (html_report)


    output:
        path "report/CogentDS_analysis*.rds", emit: gene_object
        path "report/CogentDS_preliminary-analysis_report.html", emit: html_report
        path "report/CogentDS_isoform_only_analysis*.rds", emit: isoform_object, optional: true
        path "report/"
        path "count_matrices/*.csv", optional: true
        path "count_matrices_umionly/*.csv", optional: true

    script:

        def fusion_command = ''
        def immune_command = ''

        def isoforms_report_cmd = '--analyzeIsoformforReport FALSE'
        
        if (params.fusion){
            fusion_command = "--fusiondir fusion"
        }

        if (params.immune){
            immune_command = "--clonotypeCounts ${clonotype_counts} --clonotypeSummary ${clonotype_summary} --clonotypeMetadata ${clonotype_metadata}"
        }

        if (params.isoform_report){
            isoforms_report_cmd = '--analyzeIsoformforReport TRUE'
        }

        def rename_analysis_rds = umi_only ? "mv report/CogentDS_analysis.Rds CogentDS_analysis_umis.Rds" : ""
        def rename_isoform_rds = umi_only ? "mv report/CogentDS_isoform_only_analysis.Rds CogentDS_isoform_only_analysis_umis.Rds" : ""
        def matrix_path = umi_only ? "count_matrices_umionly" : "count_matrices"
        
        """
        export R_LIBS="./lib"

        mkdir ${matrix_path}
        
        Rscript ${cogentds_script} --pathtoCogentDSreporter ${cogentds_reporter} --analyzeStats ${analyze_stats} \
        --apSeuratObject ${counts_rds} --outPath \$PWD/report --matrixPath ${matrix_path} \
        ${isoforms_report_cmd} \
        ${fusion_command} \
        ${immune_command} 
        """
}
