nextflow.enable.dsl=2

conda_env_path = "${params.cogent_root}" + '/config/env/umi_tools.yaml'
if (params.type_of_experiment == "shasta_total_rna" && params.keep_intermediate ){
    publishdir_enabled = true
} else if (params.type_of_experiment == "shasta_total_rna" && !params.keep_intermediate ) {
    publishdir_enabled = false
} else {
    publishdir_enabled = true
}

process DEDUP_UMITOOLS {
    conda conda_env_path
    publishDir = [
        [
            path: { "${params.output_dir}" },
            mode: 'copy',
            pattern: "logs/umi_tools/*.log",
        ], 
        [
            path: { "${params.output_dir}" },
            mode: 'copy',
            pattern: "dedup_bam/*.bam",
            enabled: publishdir_enabled
        ]
    ]
    input:
        tuple val(sample_id), path(bam)

    output:
        tuple val (sample_id), path("dedup_bam/*dedup.out.bam"), emit: dedup_bam
        path("logs/umi_tools/*umitools.log"), emit: umitools_log

    script:


        def tag_file = 'UM'
        def modified_bam = 'temp_with_UM.bam'

        def perl_cmd = "perl -pe 's/^([^\\t]+)_([^\\t]+)(.+)/\\1_\\2\\3\\tUM:Z:\\2/'"

        if ( params.type_of_experiment == "smartseq_fla_umi"){
            perl_cmd = "perl -ne 's/^([^\\t]+)_([^\\t]+)(.+)/\\1_\\2\\3\\tUM:Z:\\2/; print if \$2 ne 'NNNNNNNN''"
        }

        """
        mkdir -p dedup_bam
        mkdir -p logs/umi_tools

        samtools view -h -F 256 ${bam} | ${perl_cmd} | samtools sort | samtools view -b > ${modified_bam}

        samtools index ${modified_bam}

        export PYTHONHASHSEED=0
        umi_tools dedup --random-seed=123456789 --stdin=${modified_bam} --log=logs/umi_tools/${sample_id}_umitools.log \
        --paired --no-sort-output --extract-umi-method=tag --unpaired-reads discard \
        --umi-tag=UM | samtools sort -n > dedup_bam/${sample_id}_Aligned.toTranscriptome.dedup.out.bam


        """
}
