#!/usr/bin/env python3
# Added changes to accomodate 1491: Embgenix PGT G&T for RHT

import argparse
import base64
from collections import OrderedDict
import glob
import io
from itertools import cycle, islice
import numpy as np
import matplotlib.pyplot as plt
plt.switch_backend('Agg')
import os
import pandas as pd
import seaborn
import textwrap
    
def pcen_str(value):
    myvalue = float(round((value * 100),2))
    return str(myvalue)+"%"

def split_sample_dataframe(counts_all_file, well_list_file, reads_min):
    # Counts all file from demux
    counts_df = pd.read_csv(counts_all_file, header=None)
    counts_df.columns= ['barcodes','types','counts']
    counts_df['counts'] = counts_df['counts'] * 2

    # Well list file
    wl_df = pd.read_table(well_list_file, sep='\t')
    if len(wl_df.columns) > 1:
        wl_df = wl_df[["Sample", "Barcode"]]
    else:
        wl_df = pd.read_table(well_list_file, sep=',')
        wl_df = wl_df[["Sample", "Barcode"]]

    Total_Barcodes_Identified=wl_df.shape[0]

    colnum_wl = len(wl_df.columns)

    # create List based on use of min read filters
    sample = pd.DataFrame(columns=['types', 'total', 'barcode'])
    sample_xk = pd.DataFrame(columns=['types', 'total', 'barcode'])
    sample_50k = 0

    pos_ctrl = pd.DataFrame(columns=['types', 'total', 'barcode'])
    pos_ctrl_xk = pd.DataFrame(columns=['types', 'total', 'barcode'])
    posctrl_50k = 0
    
    neg_ctrl = []

    for i in range(0, Total_Barcodes_Identified):

        # Barcode 
        barcode = wl_df.iloc[i,colnum_wl-1].replace('+', '')

        # Sample name
        ctrl_type = wl_df.iloc[i, colnum_wl - 2].replace(' ', '_').replace('-', '_')
        idx = counts_df.index[counts_df['barcodes']==barcode]

        if len(idx) == 1 and idx[0] >= 0 and ctrl_type == counts_df.iloc[idx[0],1]:
            if ctrl_type == 'Pos_Ctrl':
                pos_ctrl.loc[len(pos_ctrl.index)]= [ctrl_type, counts_df.iloc[idx[0], 2], counts_df.iloc[idx[0], 0] ]

                if counts_df.iloc[idx[0], 2] >= reads_min:

                    pos_ctrl_xk.loc[len(pos_ctrl_xk.index)]= [ctrl_type, counts_df.iloc[idx[0], 2], counts_df.iloc[idx[0], 0] ]
                    posctrl_50k += 1

            elif ctrl_type == 'Neg_Ctrl':
                neg_ctrl.append( counts_df.iloc[idx[0], 2] )

            else: # ctrl_type == 'sample':
                sample.loc[len(sample.index)]= [ctrl_type, counts_df.iloc[idx[0], 2], counts_df.iloc[idx[0], 0] ]
                if counts_df.iloc[idx[0], 2] >= reads_min:
                    sample_xk.loc[len(sample_xk.index)]= [ctrl_type, counts_df.iloc[idx[0], 2], counts_df.iloc[idx[0], 0] ]
                    sample_50k += 1
        else:
            if ctrl_type == 'Pos_Ctrl':
                pos_ctrl.loc[len(pos_ctrl.index)]= [ctrl_type, 0, barcode ]

            elif ctrl_type == 'Neg_Ctrl':
                neg_ctrl.append(0)
            else:
                sample.loc[len(sample.index)]= [ctrl_type, 0, barcode ]
    
    sample = sample.sort_values(by=["total", "types"])
    return(sample, neg_ctrl, pos_ctrl, sample_xk, pos_ctrl_xk)

def experimental_overview(file_name, counts_all_file, well_list_file, reads_min):
    overview = {}
    counts_df = pd.read_csv(counts_all_file, header=None)
    counts_df.columns= ['barcodes','types','counts']
    counts_df['counts'] = counts_df['counts'] * 2
    overview['total_reads_pair'] = counts_df['counts'].sum() / 2
    overview['total_reads'] = counts_df['counts'].sum()
    
    wl_df = pd.read_table(well_list_file, sep='\t')
    if len(wl_df.columns) > 1:
        wl_df = wl_df[["Sample", "Barcode"]]
    else:
        wl_df = pd.read_table(well_list_file, sep=',')
        wl_df = wl_df[["Sample", "Barcode"]]

    overview['total_barcodes_identified']=wl_df.shape[0]

    sample, neg_ctrl, pos_ctrl, sample_xk, pos_ctrl_xk = split_sample_dataframe(counts_all_file, well_list_file, reads_min)

    sample = sample.sort_values(by=["total", "types"])
    overview['total_barcoded_reads'] = sum(sample.iloc[:,1]) + sum(pos_ctrl.iloc[:,1]) + sum(neg_ctrl) 
    overview['barcoded_read_fraction'] = (overview['total_barcoded_reads'] / overview['total_reads']) * 100
    overview['min_reads_filter'] = reads_min / 2
    
    #formatting for html tbl 
    overview_tbl ={}
    overview_tbl["Total Read pairs"] = format(round(overview['total_reads_pair']),",")
    overview_tbl["Total Reads"] = format(overview['total_reads'],",")
    overview_tbl["Total Barcoded Reads"] = format(overview['total_barcoded_reads'],",")
    overview_tbl["Barcoded Reads Fraction"] = str(round(overview['barcoded_read_fraction'], 2))+"%"
    overview_tbl["Total Barcodes Identified"] = format(overview['total_barcodes_identified'],",")
    overview_tbl["Min Read Pairs Filter"] = format(round(overview['min_reads_filter']),",")

    gen_overview_tmpl(file_name, "Experimental Overview", overview_tbl)
    return(overview)

def gen_overview_tmpl(filename, title, mymap):
    if len(mymap) == 0:
        print("empty mymap")
        return
    
    with open(filename, "a") as f:
        if len(title)>0:
            f.write('\n<h2>'+title+'</h2>\n')

            colors = ['background-color:#E0E0E0','background-color:#FFFFFF']
            f.write('<table border-top:1px;border-bottom:1px cellspacing="0" cellpadding="0" >\n')
            for i, (key, val) in enumerate(mymap.items()):
                f.write("<tr style="+colors[i%2]+">")
                f.write("<td style='padding-left:10px;padding-right:10px;padding-top:2px;padding-botton:2px'>"+(key)+"</td>")
                f.write("<td style='padding-left:10px;padding-right:10px;padding-top:2px;padding-botton:2px'>"+str(val)+"</td>")
                f.write('</tr>\n')  
            f.write("</table>\n")

    return()

def Reads_by_Sample_Type(args, file_name, counts_all_file, well_list_file, reads_min, Total_Barcoded_reads):

    sample, neg_ctrl, pos_ctrl, sample_xk, pos_ctrl_xk = split_sample_dataframe(counts_all_file, well_list_file, reads_min)

    args.build_in = True

    # list of sample/cell types
    unique = list(set(sample.iloc[:,0]))
    unique.sort()
    print("Number of unique sample types = ", str(len(unique)))
    print(unique)

    box_title = "Reads by Sample Type"
    #Add boxplot to HTML
    try:
        with open(file_name, "a") as f:
        # Generate boxplot
            data_uri = generate_boxplot(args, pos_ctrl.iloc[:, 1], neg_ctrl, sample, unique)
            
            # Write the title
            f.write(f'\n<h2>{box_title}</h2>\n')
            
            # If in build_in mode, embed the base64 image, else link to saved image
            if args.build_in:
                img_tag = f'\n<img src="data:image/png;base64,{data_uri}">'
                f.write(img_tag)
            else:
                f.write('\n<img src="boxplot.png" />\n')
        
    except Exception as e:
        print(f"Error during file writing or boxplot generation: {e}")

    types, sample_id, sample_reads, stypes_pcen, sample_median, sample_mean, sample_std = [], [], [], [], [], [], []

    for i in range(len(unique)):
        sample_sub = sample[sample['types'] == unique[i]]
        sample_id.append( str(len(sample_sub.iloc[:,0])) )
        sample_reads.append( format(sum(sample_sub.iloc[:,1]),",") )

        if Total_Barcoded_reads==0:
            stypes_pcen.append( pcen_str(0) )
        else:
            stypes_pcen.append( pcen_str((sum(sample_sub.iloc[:,1]))/Total_Barcoded_reads) )

        sample_median.append( format(int(np.median(sample_sub.iloc[:,1])+0.5),",") )
        sample_mean.append( format(int(np.mean(sample_sub.iloc[:,1])+0.5),",") )
        sample_std.append( format(int(np.std(sample_sub.iloc[:,1])+0.5),",") )

    items = ["Sample Types","Pos_Ctrl", "Neg_Ctrl"] + unique
    types.append(items)
    items = ["Barcodes Identified", str(len(pos_ctrl)), str(len(neg_ctrl)) ] + sample_id
    types.append(items)
    items = ["Barcodes Reads", format(sum(pos_ctrl.iloc[:,1]),","), format(sum(neg_ctrl),",") ] + sample_reads
    types.append(items)
    if Total_Barcoded_reads==0:
        items = ["% of Total Barcoded Reads", pcen_str(0), pcen_str(0) ] + stypes_pcen
    else:
        items = ["% of Total Barcoded Reads", pcen_str((sum(pos_ctrl.iloc[:,1]))/Total_Barcoded_reads), pcen_str((sum(neg_ctrl))/Total_Barcoded_reads) ] + stypes_pcen
    types.append(items)

    median_p = mean_p = stdev_p = median_n = mean_n = stdev_n = 0

    if len(pos_ctrl) > 0:
        median_p = np.median(pos_ctrl.iloc[:,1])
        mean_p = np.mean(pos_ctrl.iloc[:,1])
        stdev_p = np.std(pos_ctrl.iloc[:,1])

    if len(neg_ctrl) > 0:
        median_n = np.median(neg_ctrl)
        mean_n = np.mean(neg_ctrl)
        stdev_n = np.std(neg_ctrl)

    items = ["Reads per Barcode_median", format(int(median_p+0.5),","), format(int(median_n+0.5),",") ] + sample_median
    types.append(items)
    items = ["Reads per Barcode_mean", format(int(mean_p+0.5),","), format(int(mean_n+0.5),",") ] + sample_mean
    types.append(items)
    items = ["Reads per Barcode_stdDev", format(int(stdev_p+0.5),","), format(int(stdev_n+0.5),",") ] + sample_std
    types.append(items)

    with open(file_name, "a") as f:

        if len(types)>0:
            colors = ['background-color:#E0E0E0','background-color:#FFFFFF']
            f.write('<table border-top:1px;border-bottom:1px cellspacing="0" cellpadding="0" >\n')
            for i in range(0, len(types)):
                row = types[i]
                for j in range(0, len(row)):
                    item = row[j]
                    if j == 0:
                        f.write("<tr style="+colors[i%2]+">")
                    f.write("<td style='padding-left:10px;padding-right:10px;padding-top:2px;padding-botton:2px'>"+str(item)+"</td>")
                    if j==len(row)-1:
                        f.write('</tr>\n')
            f.write("</table>\n")
    
    return()

def generate_boxplot(p_args, pos_ctrl, neg_ctrl, sample, unique):
    p_args.build_in = True
    seaborn.set(font_scale=1)
    seaborn.set_style('darkgrid')  # darkgrid white
    np.random.seed(10)
    
    # Process control data
    pos_ctrl = [x // 2 for x in pos_ctrl]
    neg_ctrl = [x // 2 for x in neg_ctrl]
    data = [pos_ctrl, neg_ctrl]
    types = ['Pos_Ctrl', 'Neg_Ctrl']
    
    # Add sample data
    for i in range(len(unique)):
        sample_sub = list(sample[sample.types.str.contains(unique[i])].iloc[:, 1])
        sample_sub = [x // 2 for x in sample_sub]
        data.append(sample_sub)
        types.append(unique[i])
    
    # Set figure width based on unique values
    width = 2 + len(unique) * 2
    fig = plt.figure(figsize=(width, 4))
    rate = [0.15, 0.1, 0.8, 0.8]
    if width > 12:
        rate = [0.075, 0.1, 0.9, 0.8]
    
    ax = fig.add_axes(rate)
    bp = ax.boxplot(data, patch_artist=True, notch=True, vert=90)
    ax.set_xticklabels(types)
    
    # Assign colors to boxes
    colors = ['#005a79', '#24a53b', '#4b7321', '#7cc400', '#6685db', '#009eff', '#42135d', '#04008a', '#1c82e0', '#6e0082']
    colors = colors[:2 + len(unique)]
    
    for patch, color in zip(bp['boxes'], colors):
        patch.set_facecolor(color)
    
    plt.ylabel('Reads Per Barcode')
    plt.title('Reads distribution for sample types')
    
    # Handle saving the image
    decoded = ""
    if p_args.build_in:
        img = io.BytesIO()
        fig.savefig(img, format='png', bbox_inches='tight')
        img.seek(0)
        decoded = base64.b64encode(img.getvalue()).decode('utf-8')
    else:
        fig.savefig(os.path.join(p_args.out_dir, "boxplot.png"))
    
    # Clean up and close figure
    plt.close(fig)
    
    return decoded

def chars_in_ATGC(seg):
    ATGClist = ["A", "T", "G", "C"]
    for i in range( len(seg)):
        if not seg[i] in ATGClist:
            return False
    return True

def filter_out_by_barcodes(mf_df, barcodes):
    mydf = pd.DataFrame(columns=mf_df.columns)
    for nrow in range( len(mf_df)):
        segs = mf_df["Sample"][nrow].split("_")
        inx_barcode = -1
        for i in range( len(segs) ):
            if len(segs[i]) == 16 and chars_in_ATGC(segs[i]):
                inx_barcode = i
                break
        barcode = segs[inx_barcode]
        if barcode in barcodes:
            mydf.loc[len(mydf.index)]= mf_df.loc[nrow]
    return mydf

def find_pos_ctrl_reads(df):
    df = df.loc[df['Sample'].str.contains('bowtie2_sorted')]
    df = df.loc[df['Sample'].str.contains('Pos')]
    pos_dup_reads_sub = cal_dup_reads_from_picard(df)
    pos_sample_dup_reads = format(pos_dup_reads_sub,",")
    return(pos_sample_dup_reads)

def find_pos_ctrl_percent(df):
    df = df.loc[df['Sample'].str.contains('bowtie2_sorted')]
    df = df.loc[df['Sample'].str.contains('Pos')]
    pos_dup_reads_percent = cal_dup_from_picard(df)
    return(pcen_str(pos_dup_reads_percent))

def cal_dup_from_picard(df):
    UNPAIRED_READS_EXAMINED = df['UNPAIRED_READS_EXAMINED'].sum()
    READ_PAIRS_EXAMINED = df['READ_PAIRS_EXAMINED'].sum()
    UNPAIRED_READ_DUPLICATES = df['UNPAIRED_READ_DUPLICATES'].sum()
    READ_PAIR_DUPLICATES = df['READ_PAIR_DUPLICATES'].sum()
    TOTAL_READS = UNPAIRED_READS_EXAMINED + (2 * READ_PAIRS_EXAMINED)
    TOTAL_DUPLICATE_READS = UNPAIRED_READ_DUPLICATES + (2 * READ_PAIR_DUPLICATES)
    DUPLICATION_RATE = (TOTAL_DUPLICATE_READS/TOTAL_READS)
    return(DUPLICATION_RATE)

def cal_dup_reads_from_picard(df):
    UNPAIRED_READ_DUPLICATES = df['UNPAIRED_READ_DUPLICATES'].sum()
    READ_PAIR_DUPLICATES = df['READ_PAIR_DUPLICATES'].sum()
    TOTAL_DUPLICATE_READS = UNPAIRED_READ_DUPLICATES + (2 * READ_PAIR_DUPLICATES)
    TOTAL_DUPLICATE_READS = TOTAL_DUPLICATE_READS 
    return(TOTAL_DUPLICATE_READS)

def check_total_reads_reported(df):
    df = df.loc[df['Sample'].str.contains('bowtie2_sorted')]
    df = df.loc[df['Sample'].str.contains('Pos')]
    UNPAIRED_READS_EXAMINED = df['UNPAIRED_READS_EXAMINED'].sum()
    READ_PAIRS_EXAMINED = df['READ_PAIRS_EXAMINED'].sum()
    TOTAL_READS = UNPAIRED_READS_EXAMINED + (2 * READ_PAIRS_EXAMINED)
    return(TOTAL_READS)

def cal_total_reads_from_picard(df):
    UNPAIRED_READS_EXAMINED = df['UNPAIRED_READS_EXAMINED'].sum()
    READ_PAIRS_EXAMINED = df['READ_PAIRS_EXAMINED'].sum()
    TOTAL_READS = UNPAIRED_READS_EXAMINED + (2 * READ_PAIRS_EXAMINED)
    TOTAL_READS = TOTAL_READS 
    return(TOTAL_READS)

# Function for section 3
def read_stat_filtered(file_name, PicoPLEX_RD_FOLDER, counts_all_file, well_list_file, reads_min):

    # MultiQC - FASTQC dataframe
    fastqc_file = glob.glob(f"{PicoPLEX_RD_FOLDER}/**/*multiqc_fastqc.txt", recursive=True)
    fastqc_df = pd.read_table(fastqc_file[0])
    fastqc_df['SampleID'] = fastqc_df.Sample.str.split(r'_[ACGT]+_(?:trimmed_)?R2').str[0]
    print(fastqc_file)
    sample, neg_ctrl, pos_ctrl, sample_xk, pos_ctrl_xk = split_sample_dataframe(counts_all_file, well_list_file, reads_min)
    sample_filter = sample.loc[sample['total'] >= reads_min]
    pos_ctrl_filter = list(pos_ctrl_xk["total"])
    barcodes = list(sample_xk["barcode"]) + list(pos_ctrl_xk["barcode"])
    fastqc_df = filter_out_by_barcodes(fastqc_df, barcodes)
    mask = fastqc_df.Sample.str.contains("_trimmed_")
    df_Nontrimmed = fastqc_df[~mask]
    df_trimmed = fastqc_df[mask]
    df_trimmed.to_csv('out-sample-trimmed.csv', index=False)
    mask_Nontrimmed_p = df_Nontrimmed.Sample.str.contains("Pos_")
    df_Nontrimmed_posctrl = df_Nontrimmed[mask_Nontrimmed_p]
    mask_Nontrimmed_pn = df_Nontrimmed.Sample.str.contains("Pos_|Neg_")
    df_Nontrimmed_sample = df_Nontrimmed[~mask_Nontrimmed_pn]
    mask_trimmed_p = df_trimmed.Sample.str.contains("Pos_")
    df_trimmed_posctrl = df_trimmed[mask_trimmed_p]
    mask_trimmed_pn = df_trimmed.Sample.str.contains("Pos_|Neg_")
    df_trimmed_sample = df_trimmed[~mask_trimmed_pn]
    barcoded_s_reads = int(df_Nontrimmed_sample["Total Sequences"].sum())
    barcoded_p_reads = int(df_Nontrimmed_posctrl["Total Sequences"].sum())
    trimmed_s_reads = int(df_trimmed_sample["Total Sequences"].sum())
    trimmed_p_reads = int(df_trimmed_posctrl["Total Sequences"].sum()) 

    # Populate HTML with following
    types = []

    # A) Sample Types (from multi-QC)
    unique = list(set(sample.iloc[:,0]))
    unique.sort()
    items = ["Sample Types","Pos_Ctrl",""]
    for i in range(len(unique)):
        items = items + [unique[i], ""]
    types.append(items)

    # B) Passing-filter Barcodes (from multi-QC)
    sample_idfilter, sample_reads, sample_reads_trimmed = [], [], []

    for i in range(len(unique)):
        sample_sub = sample_filter[ sample_filter['types'] == unique[i] ]
        sample_subfilter = sample_sub[sample_sub['total'] >= reads_min]
        sample_idfilter.append( format(len(sample_subfilter.iloc[:,0]),",") )
        sample_idfilter.append("")
    
    items = ["Passing-filter Barcodes", str(len(pos_ctrl_filter)),""] + sample_idfilter
    types.append(items)

    items = ["", "Reads", "% Barcoded Reads", "Reads", "% Barcoded Reads"]
    for i in range(1,len(unique)):
        items.append("Reads")
        items.append("% Barcoded Reads")
    types.append(items)

    # C) Barcoded Reads (from multi-QC) and D) Trimmed Reads (from multi-QC)
    for i in range(len(unique)):
        sample_sub = sample_filter[ sample_filter['types'] == unique[i] ]
        sample_subfilter = sample_sub[sample_sub['total'] >= reads_min]
        sample_barcoded_reads_sub = sum(sample_subfilter.iloc[:,1])
        sample_reads.append( format(sample_barcoded_reads_sub,",") )
        
        if sample_barcoded_reads_sub==0:
            sample_reads.append( "0.00%" )
        else:
            sample_reads.append( "100.00%" )

        mark_sample = df_trimmed_sample.SampleID.str.contains(unique[i])
        df_trimmed_sample_sub = df_trimmed_sample[mark_sample]
        trimmed_reads_sub = int(df_trimmed_sample_sub["Total Sequences"].sum())
        sample_reads_trimmed.append( format(trimmed_reads_sub,",") )
        
        if sample_barcoded_reads_sub==0:
            sample_reads_trimmed.append( pcen_str(0) )
        else:
            sample_reads_trimmed.append( pcen_str(trimmed_reads_sub/sample_barcoded_reads_sub) )

    if len(pos_ctrl_filter) > 0:
        items = ["Barcoded Reads", format(sum(pos_ctrl_filter),","), "100.00%"] + sample_reads
        types.append(items)
        items = ["Trimmed Reads", format(trimmed_p_reads,","), pcen_str(trimmed_p_reads/(sum(pos_ctrl_filter)))] + sample_reads_trimmed
        types.append(items)
    else:
        items = ["Barcoded Reads", format(sum(pos_ctrl_filter),","), "0%"] + sample_reads
        types.append(items)
        items = ["Trimmed Reads", format(trimmed_p_reads,","), pcen_str(0)] + sample_reads_trimmed
        types.append(items)

    # E) Unmapped Reads (from picard) and  F) Mapped Reads (from Picard)

    sample_unmap_reads, sample_map_reads, sample_dup_reads, sample_mapunique_reads = [], [], [], []
    
    # Picard dataframe
    picard_file = glob.glob(f"{PicoPLEX_RD_FOLDER}/**/*multiqc_picard_dups.txt", recursive=True)
    picard_df = pd.read_table(picard_file[0])
    picard_df['SampleID'] = picard_df.Sample.str.split(r'_[ACGT]+_trimmed_bowtie2_sorted').str[0]
    picard_df = filter_out_by_barcodes(picard_df, barcodes)
    mark = picard_df.Sample.str.contains("sorted")
    df_all = picard_df[mark]
    mark_pos = df_all.Sample.str.contains("Pos")
    df_all_p = df_all[mark_pos]
    mark_pn = df_all.Sample.str.contains("Pos|Neg")
    df_all_s = df_all[~mark_pn]
    unmap_s_reads = int(df_all_s["UNMAPPED_READS"].sum())
    unmap_p_reads = int(df_all_p["UNMAPPED_READS"].sum())
    map_s_reads = int(df_all_s["UNPAIRED_READS_EXAMINED"].sum()+2*df_all_s["READ_PAIRS_EXAMINED"].sum() )
    map_p_reads = int(df_all_p["UNPAIRED_READS_EXAMINED"].sum()+2*df_all_p["READ_PAIRS_EXAMINED"].sum() )
    multimap_s_reads = int(df_all_s["UNPAIRED_READ_DUPLICATES"].sum()+2*df_all_s["READ_PAIR_DUPLICATES"].sum() )
    multimap_p_reads = int(df_all_p["UNPAIRED_READ_DUPLICATES"].sum()+2*df_all_p["READ_PAIR_DUPLICATES"].sum() )
    uniquemap_s_reads = map_s_reads - multimap_s_reads
    uniquemap_p_reads = map_p_reads - multimap_p_reads

    for i in range(len(unique)):
        sample_sub = sample_filter[ sample_filter['types'] == unique[i] ]
        sample_subfilter = sample_sub[sample_sub['total'] >= reads_min]
        sample_barcoded_reads_sub = sum(sample_subfilter.iloc[:,1])
        df_all_s_sub = df_all_s[df_all_s.SampleID.str.contains(unique[i])]

        # Unmapped reads
        unmap_reads_sub = int(df_all_s_sub["UNMAPPED_READS"].sum())
        sample_unmap_reads.append( format(unmap_reads_sub,",") )

        if sample_barcoded_reads_sub==0:
            sample_unmap_reads.append( pcen_str(0) )
        else:
            sample_unmap_reads.append( pcen_str(unmap_reads_sub/sample_barcoded_reads_sub) )

        # Mapped reads
        map_reads_sub = cal_total_reads_from_picard(df_all_s_sub)
        sample_map_reads.append( format(map_reads_sub,",") )

        if sample_barcoded_reads_sub==0:
            sample_map_reads.append( pcen_str(0) )
        else:
            sample_map_reads.append( pcen_str(map_reads_sub/sample_barcoded_reads_sub) )

        # Duplicate reads
        dup_reads_sub = cal_dup_reads_from_picard(df_all_s_sub)
        sample_dup_reads.append( format(dup_reads_sub,",") )
        
        if sample_barcoded_reads_sub==0:
            sample_dup_reads.append( pcen_str(0) )
        else:
            sample_dup_reads.append( pcen_str(dup_reads_sub/sample_barcoded_reads_sub) )

        # Unique Reads
        dup_reads_sub = cal_dup_reads_from_picard(df_all_s_sub)
        uniquemap_reads_sub = map_reads_sub - dup_reads_sub
        sample_mapunique_reads.append( format(uniquemap_reads_sub,",") )

        if sample_barcoded_reads_sub==0:
            sample_mapunique_reads.append( pcen_str(0) )
        else:
            sample_mapunique_reads.append( pcen_str(uniquemap_reads_sub/sample_barcoded_reads_sub) )

    # HTML file populate section 3 out of 6

    title = 'Reads Statistics wells after Min Reads Filter'

    if unmap_p_reads == 0:
        items = ["&nbsp;&nbsp;Unmapped Reads", format(unmap_p_reads,","), "0%"] + sample_unmap_reads
        
    else:
        items = ["&nbsp;&nbsp;Unmapped Reads", format(unmap_p_reads,","), pcen_str(unmap_p_reads/(sum(pos_ctrl_filter)))] + sample_unmap_reads
    types.append(items)

    if map_p_reads == 0:
        items = ["&nbsp;&nbsp;Mapped Reads", format(map_p_reads,","), "0%"] + sample_map_reads
    else:
        items = ["&nbsp;&nbsp;Mapped Reads", format(map_p_reads,","), pcen_str(map_p_reads/(sum(pos_ctrl_filter)))] + sample_map_reads
    types.append(items)
        
    if multimap_p_reads == 0:
        items = ["&nbsp;&nbsp;&nbsp;&nbsp;Duplicate Reads", format(multimap_p_reads,","), "0%"] + sample_dup_reads
    else:
        items = ["&nbsp;&nbsp;&nbsp;&nbsp;Duplicate Reads", format(multimap_p_reads,","), pcen_str(multimap_p_reads/sum(pos_ctrl_filter))] + sample_dup_reads
    types.append(items)
    if uniquemap_p_reads == 0:
        items = ["&nbsp;&nbsp;&nbsp;&nbsp;Unique Reads", format(uniquemap_p_reads,","), "0%"] + sample_mapunique_reads
    else:
        items = ["&nbsp;&nbsp;&nbsp;&nbsp;Unique Reads", format(uniquemap_p_reads,","), pcen_str(uniquemap_p_reads/sum(pos_ctrl_filter))] + sample_mapunique_reads
    types.append(items)

    with open(file_name, "a") as f:
        if len(title)>0:
            f.write('\n<h2>'+title+'</h2>\n')

        if len(types)>0:
            colors = ['background-color:#E0E0E0','background-color:#FFFFFF']
            f.write('<table border-top:1px;border-bottom:1px cellspacing="0" cellpadding="0" >\n')
            for i in range(0, len(types)):
                row = types[i]
                for j in range(0, len(row)):
                    item = row[j]
                    if j == 0:
                        f.write("<tr style="+colors[i%2]+">")
                    f.write("<td style='padding-left:10px;padding-right:10px;padding-top:2px;padding-botton:2px'>"+str(item)+"</td>")
                    if j==len(row)-1:
                        f.write('</tr>\n')
            f.write("</table>\n")

    return()

def dbg_print(mymap):
    for k, v in mymap.items():
        print(k +": ", v)

def get_insert_info(p_args, barcodes):
    df_insert = pd.read_table(os.path.join(p_args.PicoPLEX_RD,"multiqc_report_data", "multiqc_picard_insertSize.txt"))
    df_insert = df_insert[["Sample","MEDIAN_INSERT_SIZE","MEAN_INSERT_SIZE"]]
    df_insert = filter_out_by_barcodes(df_insert, barcodes)
    df_insert = df_insert.astype({"MEDIAN_INSERT_SIZE":"int32", "MEAN_INSERT_SIZE":"float32"})
    
    mask = df_insert.Sample.str.contains("Pos_")
    df_insert_p = df_insert[mask]
    df_insert_p = df_insert_p.reset_index(drop=True)
    mask_pn = df_insert.Sample.str.contains("Pos_|Neg_")
    df_insert_s = df_insert[~mask_pn]
    df_insert_s = df_insert_s.reset_index(drop=True)
    
    mean_p = ""
    median_p = ""
    if len(df_insert_p) > 0:
        median_p = dicimal_str( int(100*np.median(df_insert_p.iloc[:,1])+0.5)/100, 2)
        mean_p = dicimal_str( int(100*np.mean(df_insert_p.iloc[:,2])+0.5)/100, 2)
    
    unique = []
    df_insert_s_type = pd.DataFrame(columns=["type", "median", "mean"]) 
    for i in range(len(df_insert_s)):
        ctrl_type = get_ctrl_type( df_insert_s["Sample"][i] )
        unique.append(ctrl_type)
        df_insert_s_type.loc[len(df_insert_s_type.index)] = [ctrl_type, df_insert_s.iloc[i,1], df_insert_s.iloc[i,2] ]
    unique = list(set(unique))
    unique.sort()
    mean_s = []
    median_s = []
    for i in range(len(unique)):
        df_insert_s_sub = df_insert_s_type[df_insert_s_type['type'] == unique[i]]
        median_s.append( dicimal_str( int(100*np.median(df_insert_s_sub.iloc[:,1])+0.5)/100, 2) )
        mean_s.append( dicimal_str( int(100*np.mean(df_insert_s_sub.iloc[:,2])+0.5)/100, 2) )
    return mean_p, median_p, unique, mean_s, median_s

def dicimal_str(value, num=2):  #use float to str
    str_c = str(value)
    pos = str_c.find('.')
    if pos >= 0 and pos > len(str_c)-num-1:
        for i in range(len(str_c)-num-1, pos):
            str_c = str_c + ("0")
    return str_c

def get_ctrl_type(filepath):
#pre: sample type should be from start of basename to barcod
    segs = os.path.basename(filepath).split("_")
    sample_type = segs[0]
    inx_barcode = -1
    for i in range( len(segs) ):
        if len(segs[i]) == 16 and chars_in_ATGC(segs[i]):
            inx_barcode = i
            break
    if segs[0] == "demux" and inx_barcode == 1 and len(segs)>3:
        # sample_type should be after barcode
        sample_type = segs[2]
        inx_end = 3
        for i in range(inx_end, len(segs)):
            if (segs[i][0].isnumeric() and segs[i][1] == "M") or segs[i]=="trimmed":
                break
            else:
                sample_type += "_" + segs[i]
    else:
        for i in range(1, inx_barcode):
            sample_type += "_" + segs[i]
    return sample_type

def histogram_plot_chromosome(args, chrs_p, chrs_s, unique):
    args.build_in = True
    seaborn.set() #make the plots look pretty
    mydf = pd.DataFrame()
    n_chr = max([len(c) for c in chrs_s])
    n_chr = max(n_chr, len(chrs_p))
    mylist = [i for i in range(1, n_chr+1)]
    mydf['Chromosome'] = mylist
    if len(chrs_p) > 0:
        mydf["Pos_Ctrl"] = chrs_p
    for i in range(len(unique)):
        mydf[unique[i]] = chrs_s[i]
    mydf = mydf.set_index('Chromosome')
    width = 6 + len(unique) #8
    max_len = 6
    if len(unique) > 0:
        max_len = max([len(i) for i in unique])
    numcol = int( ( 1 + len(unique) )/ (max_len/10) + 1 )# (2+len(unique))//2
    # mycolors = list(islice(cycle(['b', 'r', 'g', 'y', 'k']), None, len(mydf)))
    mycolors = list(islice(cycle(['#009eff', '#42135d', '#04008a', '#1c82e0', '#6e0082']), None, len(mydf)))
    res=mydf.plot(kind='bar', figsize=(width,5), width = 0.85, color=mycolors).legend(loc=(0,1.02),ncol=numcol).get_figure()#fontsize=26,
    decoded = ""
    if args.build_in :
        img = io.BytesIO()
        f=res.savefig(img, format='png', bbox_inches='tight',)
        img.seek(0)
        decoded = base64.b64encode(img.getvalue()).decode('utf-8')
    else:
        f=res.savefig(os.path.join("histogram.png"))
    #plt.show()        
    #plt.close(f)
    return(decoded)

def get_barcode(filepath):
#pre: sample type should be from start of basename to barcode
    segs = os.path.basename(filepath).split("_")
    sample_type = segs[0]
    inx_barcode = -1
    for i in range( len(segs) ):
        if len(segs[i]) == 16 and chars_in_ATGC(segs[i]):
            inx_barcode = i
            break
    barcode = ""
    if inx_barcode >= 0:
        barcode = segs[inx_barcode]
    return barcode 

def filter_out_columns(df, barcodes):
    colnames = df.columns[3:]
    colnames_xk = list(df.columns[:3])
    for i in range( len(colnames) ):
        colname = colnames[i]
        barcode = get_barcode(colname)
        if barcode in barcodes:
            colnames_xk.append(colname)
    mydf = df[colnames_xk]
    return mydf

def get_df_sid_barcode(df, sample_barcode_fname, req_colname="Sample", req_head=False, head_posstart=1):
    tempdf_barcode = pd.read_table( sample_barcode_fname )
    sids = list(tempdf_barcode["Sample_Name"])
    if len(req_colname) > 0:
        for i in range(len(df)):
            segs = df[req_colname][i].replace("'","").split("_")
            if segs[0]+"_"+segs[1] in sids:
                index = sids.index(segs[0]+"_"+segs[1])
                barcode = tempdf_barcode["Barcode"][index]
                head = segs[0] + "_" +barcode
                for j in range(2, len(segs)):
                    head = head + "_" + segs[j]
                df.loc[i, req_colname] = head
    if req_head:
        columns = list(df.columns)
        for i in range(head_posstart,len(columns)):
            segs = columns[i].replace("'","").split("_")
            if segs[0]+"_"+segs[1] in sids:
                index = sids.index(segs[0]+"_"+segs[1])
                barcode = tempdf_barcode["Barcode"][index]
                head = segs[0] + "_" +barcode
                for j in range(2, len(segs)):
                    head = head + "_" + segs[j]
                columns[i] = head
        df.columns = columns
    return df

def get_rowsum_reads_bytypes(df_mct_iloc, pos_inxs, neg_inxs, unique, sample_inxs):
    rowsum_p_reads = 0
    rowsum_n_reads = 0
    rowsum_s_reads = [ 0 for _ in range(len(unique)) ]
    for j in range(3, len(df_mct_iloc)):
        count = 0
        find_in_sample = False
        while count < len(sample_inxs) and find_in_sample == False:
            if j in sample_inxs[count]:
                rowsum_s_reads[count] += int(df_mct_iloc.iloc[j])
                find_in_sample = True
            count += 1
        if find_in_sample == False and j in pos_inxs:
            rowsum_p_reads += int(df_mct_iloc.iloc[j])
        if find_in_sample == False and j in neg_inxs:
            rowsum_n_reads += int(df_mct_iloc[j])
    return rowsum_p_reads, rowsum_n_reads, rowsum_s_reads


def heatmapplot(p_args, df_corr, unique, scale=1):
    colnum = len(df_corr)#70
    leng = 70 #int(colnum/3.5)
    high = int(11*leng/14)
    fontsize = 18
    padsize = 12
    if scale != 1:
        leng = int(leng/5)
        high = int(high/5)
        fontsize = 2
        padsize = 2
    
    df = df_corr.drop(df_corr.iloc[:, colnum:],axis = 1)
    if colnum < len(df_corr):
        df = df[:-len(df)+colnum]
    df_corr = df.copy()
    if len(df_corr) <= colnum:
        plt.figure(figsize=(leng, high))
        seaborn.set(font_scale= scale)
        heatmap = seaborn.heatmap(df_corr, vmin=0, vmax=1, cmap='seismic') #cmap='RdBu_r'
        # heatmap = seaborn.heatmap(df_corr, vmin=0, vmax=1, cmap='BrBG') #cmap='RdBu_r'
        heatmap.set_title('Sample Types Correlation Heatmap', fontdict={'fontsize':fontsize}, pad= padsize )
    else:    
        seaborn.heatmap(df_corr)
    decoded = ""
    if scale == 1:
        f = plt.savefig( os.path.join('heatmap.png'), bbox_inches='tight' )
    else: # scale==0.5
        img = io.BytesIO()
        f=plt.savefig(img, format='png', bbox_inches='tight',)
        img.seek(0)
        decoded = base64.b64encode(img.getvalue()).decode('utf-8')
    plt.close(f)
    seaborn.set(font_scale= 1)
    return decoded

def get_key_name(namestr, unique):
    item = get_ctrl_type(namestr)
    segs = namestr.split("_")
    inx_barcode = -1
    for i in range( len(segs) ):
        if len(segs[i]) == 16 and chars_in_ATGC(segs[i]):
            inx_barcode = i
            break
    barcode = segs[inx_barcode]
    if namestr.find("Pos") >= 0:
        item = "0_" + barcode
    elif namestr.find("Neg") >= 0:
        item = "1_" + barcode
    elif unique.index(item) >= 0: 
        item = str( 2+unique.index(item) ) + "_" + barcode
    return item

def arrange_celltypes(p_args, df_pear, unique, barcodes):
    temp_mat = df_pear.stack()
    fd_mat = temp_mat.reset_index()
    fd_mat.columns = ['level_0', 'level_1', 0]
    filter = fd_mat['level_0'].str.contains("Neg")
    fd_mat = fd_mat[~filter]
    filter = fd_mat['level_1'].str.contains("Neg")
    fd_mat = fd_mat[~filter]
    fd_mat = fd_mat.reset_index(drop=True)
    rows_del = []
    for i in range( len(fd_mat) ):
        barcode0 = get_barcode( fd_mat['level_0'][i] )
        barcode1 = get_barcode( fd_mat['level_1'][i] )
        if barcode0 not in barcodes or barcode1 not in barcodes:
            rows_del.append(i)
    fd_mat = fd_mat.drop(rows_del)
    fd_mat = fd_mat.reset_index(drop=True)
    namemap_dict = {}
    for i in range(len(df_pear)):
        namemap_dict[ df_pear.index[i] ] = get_key_name( df_pear.index[i], unique)
    items0 = []
    items1 = []
    for i in range(len(fd_mat)):
        item = namemap_dict[ fd_mat.iloc[i, 0] ] #get_key_name( fd_mat.iloc[i, 0], unique)
        items0.append(item)
        item = namemap_dict[ fd_mat.iloc[i, 1] ] #get_key_name( fd_mat.iloc[i, 1], unique)
        items1.append(item)
    fd_mat.insert( loc = 0, column = 'item_0', value = items0)
    fd_mat.insert( loc = 1, column = 'item_1', value = items1)
    sorted_fd_mat = fd_mat.sort_values(by=['item_0', 'item_1'])
    sorted_fd_mat = sorted_fd_mat.drop( sorted_fd_mat.columns[ [0,1] ], axis = 1 )
    sorted_fd_mat = sorted_fd_mat.set_index(['level_0', 'level_1'])
    sorted_fd_mat.index.names = [None, None]
    sorted_fd_mat = sorted_fd_mat[0]
    df_pear_sorted = build_corr_matrix(sorted_fd_mat, p_args)
    return df_pear_sorted

def build_corr_matrix(sorted_stack_mat, p_args):
    nrows = int( np.sqrt(len(sorted_stack_mat)) )
    colnames = []
    for i in range(nrows):
        colnames.append(sorted_stack_mat.index[i][1])
    df_corr = pd.DataFrame(columns = colnames)
    row = []
    for i in range(len(sorted_stack_mat)):
        row.append(sorted_stack_mat.iloc[i])
        numi = i//nrows
        numj = i%nrows
        if numi==numj and sorted_stack_mat.index[i][0] != df_corr.columns[numj]:
            print("row vs col name different")
        if i > 0 and i % nrows == nrows -1:
            df_corr.loc[len(df_corr)] = row
            row = []
    df_corr.index = df_corr.columns
    return df_corr

def pearson_boxplot(p_args, pos_ctrl, sample, unique):
    np.random.seed(10)
    data = [pos_ctrl]
    types = ['Pos_Pos']
    types_len = [4]
    for i in range (len(unique)):
        sample_sub = sample[i]
        data.append(sample_sub)
        types.append( unique[i]+"-"+unique[i] )
        types_len.append( len(unique[i])+1 )
    max_len = 8
    if len(unique) > 0:
        max_len = max([len(i) for i in unique])
    width = 2 + len(unique) + int(max_len/7)*2
    fig = plt.figure(figsize =(width, 4+int(max_len/7)))
    xpect = 1 - 1/width
    ax = fig.add_axes([0.7/width, 0.13, xpect, 0.8])
    bp = ax.boxplot(data, patch_artist = True, notch='True', vert = 90)
    ax.set_xticklabels( textwrap.fill(x, y) for x, y in zip(types,types_len) )
    # Creating plot
    colors = ['#1c82e0', '#6e0082','#005a79', '#009eff', '#42135d', '#04008a', '#24a53b', '#4b7321', '#7cc400', '#6685db']
    colors = colors[:1+len(unique)]
    
    for patch, color in zip(bp['boxes'], colors):
        patch.set_facecolor(color)
    plt.ylabel('Pearson\'s Correlation')
    plt.title("Pearson\'s Correlation for each sample type" )
    decoded = ""
    if p_args.build_in :
        img = io.BytesIO()
        f=fig.savefig(img, format='png', bbox_inches='tight',)
        img.seek(0)
        decoded = base64.b64encode(img.getvalue()).decode('utf-8')
    else:
        f = fig.savefig(os.path.join("pearson_boxplot.png"))
    fig.clear()
    plt.close(f)
    return decoded

def additional_metrics(args, file_name, PicoPLEX_RD_FOLDER, counts_all_file, well_list_file, reads_min):

    sample, neg_ctrl, pos_ctrl, sample_xk, pos_ctrl_xk = split_sample_dataframe(counts_all_file, well_list_file, reads_min)
    barcodes = list(sample_xk["barcode"]) + list(pos_ctrl_xk["barcode"])

    # --------------------
    # A) Insert Size
    # --------------------
    insertsize_file = glob.glob(f"{PicoPLEX_RD_FOLDER}/**/*multiqc_picard_insertSize.txt", recursive=True)
    print(insertsize_file)
    insert_df = pd.read_table(insertsize_file[0])
    insert_df = insert_df[["Sample","MEDIAN_INSERT_SIZE","MEAN_INSERT_SIZE"]]
    insert_df = filter_out_by_barcodes(insert_df, barcodes)
    insert_df = insert_df.astype({"MEDIAN_INSERT_SIZE":"int32", "MEAN_INSERT_SIZE":"float32"})
    
    mask = insert_df.Sample.str.contains("Pos_")
    df_insert_p = insert_df[mask]
    df_insert_p = df_insert_p.reset_index(drop=True)
    mask_pn = insert_df.Sample.str.contains("Pos_|Neg_")
    df_insert_s = insert_df[~mask_pn]
    df_insert_s = df_insert_s.reset_index(drop=True)

    insert_mean_s, insert_median_s = [], []

    insert_mean_p = ""
    insert_median_p = ""
    if len(df_insert_p) > 0:
        insert_median_p = dicimal_str( int(100*np.median(df_insert_p.iloc[:,1])+0.5)/100, 2)
        insert_mean_p = dicimal_str( int(100*np.mean(df_insert_p.iloc[:,2])+0.5)/100, 2)


    unique = []
    df_insert_s_type = pd.DataFrame(columns=["type", "median", "mean"]) 
    for i in range(len(df_insert_s)):
        ctrl_type = get_ctrl_type( df_insert_s["Sample"][i] ) #sample name
        unique.append(ctrl_type)
        df_insert_s_type.loc[len(df_insert_s_type.index)] = [ctrl_type, df_insert_s.iloc[i,1], df_insert_s.iloc[i,2] ]
    
    unique = list(set(unique))
    unique.sort()
    mean_s = []
    median_s = []
    for i in range(len(unique)):
        df_insert_s_sub = df_insert_s_type[df_insert_s_type['type'] == unique[i]]
        median_s.append( dicimal_str( int(100*np.median(df_insert_s_sub.iloc[:,1])+0.5)/100, 2) )
        mean_s.append( dicimal_str( int(100*np.mean(df_insert_s_sub.iloc[:,2])+0.5)/100, 2) )

    insert_mean_s = mean_s
    insert_median_s = median_s

    # --------------------
    # B) GC Content
    # --------------------
    fastqc_file = glob.glob(f"{PicoPLEX_RD_FOLDER}/**/*multiqc_fastqc.txt", recursive=True)
    print(fastqc_file)
    fastqc_df = pd.read_table(fastqc_file[0])
    fastqc_df = fastqc_df[["Sample","%GC"]]
    fastqc_df = filter_out_by_barcodes(fastqc_df, barcodes)
    fastqc_df = fastqc_df.astype({"%GC":"int32"})
    mask = fastqc_df.Sample.str.contains("_trimmed_")
    df_qc_trimmed = fastqc_df[mask]
    df_qc_trimmed.columns = ['types', 'value']
    df_qc_trimmed = df_qc_trimmed.reset_index(drop=True)
    for i in range(len(df_qc_trimmed)):
        df_qc_trimmed.at[i,'types'] = get_ctrl_type(df_qc_trimmed['types'][i])
    
    mask = df_qc_trimmed.types.str.contains("Pos")
    df_qc_p = df_qc_trimmed[mask]
    mask_pn = df_qc_trimmed.types.str.contains("Pos|Neg")
    df_qc_s = df_qc_trimmed[~mask_pn]

    gc_mean_p = ""
    gc_median_p = ""
    if len(df_qc_p) > 0:
        gc_mean_p = dicimal_str(int(100*np.mean(df_qc_p.iloc[:,1])+0.5)/100, 2) + "%"
        gc_median_p = dicimal_str(int(100*np.median(df_qc_p.iloc[:,1])+0.5)/100, 2) + "%"
    gc_mean_s = []
    gc_median_s = []
    for i in range(len(unique)):
        df_qc_s_sub = df_qc_s[df_qc_s['types'].str.contains(unique[i])==True] 
        gc_mean_s.append( dicimal_str(str(int(100*np.mean(df_qc_s_sub.iloc[:,1])+0.5)/100), 2) + "%" )
        gc_median_s.append( dicimal_str(str(int(100*np.median(df_qc_s_sub.iloc[:,1])+0.5)/100), 2) + "%" )

    # ----------------------------------
    # C) Normalized Reads per Chromosome
    # ----------------------------------
    countsTable_file = glob.glob(f"{PicoPLEX_RD_FOLDER}/**/*countsTable.txt", recursive=True)
    print(countsTable_file)
    countsTable_df = pd.read_table(countsTable_file[0])
    countsTable_df = filter_out_columns(countsTable_df, barcodes)
    countsTable_df = countsTable_df[countsTable_df["chr"].str.contains('_')==False]
    countsTable_df = countsTable_df.reset_index(drop=True)

    sp_colnames = ["chr", "length", "pos_reads"] + unique
    df_mct_pss = pd.DataFrame(columns=sp_colnames)

    pos_inxs = []
    neg_inxs = []
    sample_inxs = [ [] for _ in range(len(unique)) ]

    for i in range(3, len(countsTable_df.columns)):
        coltype = get_ctrl_type(countsTable_df.columns[i]) 
        if coltype.find( "Pos_" ) >= 0:
            pos_inxs.append(i)
        elif coltype.find( "Neg_" ) >= 0:
            neg_inxs.append(i)
        else: 
            inx = unique.index(coltype)
            if inx >= 0: 
                sample_inxs[inx].append(i)
    chr_s_reads = [0] * len(unique)
    chr_p_reads = 0
    chr_n_reads = 0
    for i in range(countsTable_df.shape[0]):
        rowsum_p_reads, rowsum_n_reads, rowsum_s_reads = get_rowsum_reads_bytypes(countsTable_df.loc[i], pos_inxs, neg_inxs, unique, sample_inxs)
        if i == 0 or countsTable_df.iloc[i,0] == countsTable_df.iloc[i-1,0]:
            chr_p_reads += rowsum_p_reads
            for count in range(len(unique)):
                chr_s_reads[count] += rowsum_s_reads[count]
        else:
            df_mct_pss.loc[len(df_mct_pss),:] = [countsTable_df.iloc[i-1,0], countsTable_df.iloc[i-1,2], chr_p_reads] + chr_s_reads
            chr_p_reads = rowsum_p_reads
            for count in range(len(unique)):
                chr_s_reads[count] = rowsum_s_reads[count]
    
    df_mct_pss.loc[len(df_mct_pss),:] = [countsTable_df.iloc[len(countsTable_df)-1,0], countsTable_df.iloc[len(countsTable_df)-1,2], chr_p_reads] + chr_s_reads
    total_genome_length = sum(df_mct_pss.iloc[:,1])
    total_p_reads = sum(df_mct_pss.iloc[:,2])
    total_s_reads = []
    for i in range(len(unique)):
        total_s_reads.append( sum(df_mct_pss.iloc[:,3+i]) )
    df_mct_pss_noXY = df_mct_pss.loc[(df_mct_pss['chr']!='chrX') & (df_mct_pss['chr']!='chrY')]
    df_mct_pss_noXY = df_mct_pss_noXY[df_mct_pss_noXY.chr.str.contains('[K]') == False]
    df_mct_pss_noXY = df_mct_pss_noXY[df_mct_pss_noXY.chr.str.contains('[G]') == False]
    df_mct_pss_noXY = df_mct_pss_noXY[df_mct_pss_noXY.chr.str.contains('[X]') == False]
    df_mct_pss_noXY = df_mct_pss_noXY[df_mct_pss_noXY.chr.str.contains('[Y]') == False]
    df_mct_pss_noXY = df_mct_pss_noXY[df_mct_pss_noXY.chr.str.contains('[M]') == False]
    df_mct_pss_noXY = df_mct_pss_noXY[df_mct_pss_noXY.chr.str.contains('[J]') == False]
    custom_dict = {key: f"chr{key}" for key in df_mct_pss_noXY['chr']}
    df_mct_pss_noXY = df_mct_pss_noXY.sort_values(by=['chr'], key=lambda x: x.map(custom_dict))
    df_mct_pss_noXY = df_mct_pss_noXY.reset_index(drop=True)
    df_mct_pss_noXY['chr'] = df_mct_pss_noXY['chr'].astype(int)
    df_mct_pss_noXY = df_mct_pss_noXY.sort_values(by='chr')

    chrs_p = []
    mean_p = ""
    std_p = ""
    if total_p_reads > 0:
        chrs_p = list((df_mct_pss_noXY.iloc[:,2]/total_p_reads)/(df_mct_pss_noXY.iloc[:,1]/total_genome_length))
        mean_p = dicimal_str(int(100*np.mean(chrs_p)+0.5)/100, 2)
        std_p = dicimal_str(int(100*np.std(chrs_p)+0.5)/100, 2)

    chrs_s = []
    for i in range(len(unique)):
        mylist=[]
        if total_s_reads[i]==0:
            mylist = list((df_mct_pss_noXY.iloc[:,3+i])/(df_mct_pss_noXY.iloc[:,1]/total_genome_length))
        else:
            mylist = list((df_mct_pss_noXY.iloc[:,3+i]/total_s_reads[i])/(df_mct_pss_noXY.iloc[:,1]/total_genome_length))
        chrs_s.append( mylist )
    
    mean_s = []
    std_s = []
    for i in range(len(unique)):
        mean_s.append( dicimal_str(int(100*np.mean(chrs_s[i])+0.5)/100, 2) )
        std_s.append( dicimal_str(int(100*np.std(chrs_s[i])+0.5)/100, 2) )

    chr_mean_p = mean_p
    chr_std_p = std_p
    chr_mean_s = mean_s
    chr_std_s = std_s

    # --------------------
    # D) Pearson's Correlation
    # --------------------
    pearson_matrix_file = glob.glob(f"{PicoPLEX_RD_FOLDER}/**/*pearson_matrix_noOutliers.txt", recursive=True)
    print(pearson_matrix_file)
    pearson_df = pd.read_table(pearson_matrix_file[0], skiprows=1)
    pearson_df = pearson_df.set_index("Unnamed: 0")
    pearson_df.columns = pearson_df.columns.str.replace("'", "")
    pearson_df.columns = pearson_df.columns.str.replace('"', '')
    pearson_df.index = pearson_df.index.str.replace("'", "")
    pearson_df.index = pearson_df.index.str.replace('"', '')
    pearson_df.index.name = ''
    
    correction_s = [ [] for _ in range(len(unique)) ]
    correction_p = []
    correction_n = []
    coltypes = OrderedDict()
    
    for j in range(len(pearson_df.columns)):
        coltype = get_ctrl_type(pearson_df.columns[j].replace("'",""))
        coltypes[j]=coltype
    for i in range(len(pearson_df)):
        rowtype = get_ctrl_type(pearson_df.index[i].replace("'",""))
        if rowtype.find("Pos") >= 0 : # rowtype.find("Pos") >= 0
            for j in range( len(pearson_df.columns) ):
                if (rowtype == coltypes[j]) and (i != j):
                    correction_p.append(pearson_df.iloc[i,j])
        elif rowtype.find("Neg") >= 0 : # rowtype.find("Neg") >= 0
            for j in range( len(pearson_df.columns) ):
                if (rowtype == coltypes[j]) and (i != j):
                    correction_n.append(pearson_df.iloc[i,j])     
        else: 
            for j in range( len(pearson_df.columns) ):
                if (rowtype == coltypes[j]) and (i != j) :
                    correction_s[unique.index(rowtype)].append(pearson_df.iloc[i,j]) 
    mean_p = ""
    std_p = ""
    if len(correction_p) > 0:
        mean_p = dicimal_str(int(100*np.mean(correction_p)+0.5)/100, 2)
        std_p = dicimal_str(int(100*np.std(correction_p)+0.5)/100, 2)
    mean_s = []
    std_s = []
    for j in range(len(unique)):
        mean_s.append( dicimal_str(int(100*np.mean(correction_s[j])+0.5)/100, 2) )
        std_s.append( dicimal_str(int(100*np.std(correction_s[j])+0.5)/100, 2) )

    pear_mean_p = mean_p
    pear_mean_s = mean_s
    pear_std_p = std_p
    pear_std_s = std_s

    # HTML file populate section 4 out of 6
    title = 'Additional Metrics'

    types = []
    items = ["Sample Types","Pos_Ctrl"]
    for i in range(len(unique)):
        items = items + [unique[i]]
    types.append(items)

    items = ["Insert Size (mean)", insert_mean_p ] + insert_mean_s
    types.append(items)
    items = ["Insert Size (median)", insert_median_p ] + insert_median_s
    types.append(items)

    items = ["GC Content (mean)", gc_mean_p ] + gc_mean_s
    types.append(items)
    items = ["GC Content (median)", gc_median_p ] + gc_median_s
    types.append(items)

    items = ["Normalized Reads per Chromosome (mean)", chr_mean_p ] + chr_mean_s
    types.append(items)
    items = ["Normalized Reads per Chromosome (stdev)", chr_std_p ] + chr_std_s
    types.append(items)

    items = ["Pearson's Correlation (mean)", pear_mean_p ] + pear_mean_s
    types.append(items)
    items = ["Pearson's Correlation (stdev)", pear_std_p ] + pear_std_s
    types.append(items)

    with open(file_name, "a") as f:
        if len(title)>0:
            f.write('\n<h2>'+title+'</h2>\n')

        if len(types)>0:
            colors = ['background-color:#E0E0E0','background-color:#FFFFFF']
            f.write('<table border-top:1px;border-bottom:1px cellspacing="0" cellpadding="0" >\n')
            for i in range(0, len(types)):
                row = types[i]
                for j in range(0, len(row)):
                    item = row[j]
                    if j == 0:
                        f.write("<tr style="+colors[i%2]+">")
                    f.write("<td style='padding-left:10px;padding-right:10px;padding-top:2px;padding-botton:2px'>"+str(item)+"</td>")
                    if j==len(row)-1:
                        f.write('</tr>\n')
            f.write("</table>\n")

    # Bar plot for next section
    chrom_decoded = histogram_plot_chromosome(args, chrs_p, chrs_s, unique)

    # HTML file populate section 5 out of 6
    title = 'Normalized Reads per Chromosome'
    args.build_in = True

    with open(file_name, "a") as f:
        if len(title)>0:
            f.write('\n<h2>'+title+'</h2>\n')
        if args.build_in :
            img_tag = '\n<img src="data:image/png;base64,{0}">'.format(chrom_decoded)
            f.write(img_tag)
        else:
            f.write('\n<img src=\"' + "histogram.png" + '\" />\n')

    # Last section
    df_corr = arrange_celltypes(args, pearson_df, unique, barcodes)
    pmat_decoded = heatmapplot(args, pearson_df, unique, 0.5)
    pbox_decoded = pearson_boxplot(args, correction_p, correction_s, unique)

    # HTML file populate section 6 out of 6
    title = "Pearson's Correlation"
    args.build_in = True

    with open(file_name, "a") as f:
        if len(title)>0:
            f.write('\n<h2>'+title+'</h2>\n')
        if args.build_in :
            img_tag = '\n<img src="data:image/png;base64,{0}">'.format(pmat_decoded)
            f.write(img_tag)
            img_tag = '\n<img src="data:image/png;base64,{0}">'.format(pbox_decoded)
            f.write(img_tag)
        else:
            f.write('\n<img src=\"' + "heatmap.png" + '\" width=\"700\" height=\"500\" />\n')
            f.write('\n<img src=\"' + "pearson_boxplot.png" + '\" />\n')

    return()

def main():
    parser = argparse.ArgumentParser(description='', formatter_class=lambda prog: argparse.HelpFormatter(prog, max_help_position=50)) 
    parser.add_argument('--counts_all_file', type=str, required=True, help='')
    parser.add_argument('--well_list_file', type=str, required=True, help='')
    parser.add_argument('--PicoPLEX_RD_FOLDER', type=str, required=True, help='')
    parser.add_argument('--reads_min', type=int, default=25000, required=False, help='')
    args = parser.parse_args() 

    file_name="dna_qc_metrics_report.html"
    with open(file_name, 'w') as f:
        f.write("")
           
    args.reads_min = args.reads_min * 2
    # --------------------------------------------------------
    # Section 1: Experimental Overview
    # --------------------------------------------------------
    Total_Barcoded_reads = experimental_overview(file_name, args.counts_all_file, args.well_list_file, args.reads_min)["total_barcoded_reads"]
    # --------------------------------------------------------
    # Section 2: Reads by Sample Type
    # --------------------------------------------------------
    Reads_by_Sample_Type(args, file_name, args.counts_all_file, args.well_list_file, args.reads_min, Total_Barcoded_reads)
    # --------------------------------------------------------
    # Section 3: Reads Statistics Wells above Min Reads Filter
    # --------------------------------------------------------
    read_stat_filtered(file_name, args.PicoPLEX_RD_FOLDER, args.counts_all_file, args.well_list_file, args.reads_min)
    # --------------------------------------------------------
    # Section 4, 5 and 6: Additional Metrics, Normalized Reads per Chromosome. Pearson's Correlation
    # --------------------------------------------------------
    additional_metrics(args, file_name, args.PicoPLEX_RD_FOLDER, args.counts_all_file, args.well_list_file, args.reads_min)
    

if __name__ == '__main__':
    main()