nextflow.enable.dsl=2

cogentds_script = "${params.cogent_root}" + '/lib/dna_report/CogentDS_scDNA_report.R'
cogentds_reporter = "${params.cogent_root}" + '/lib/dna_report/'
conda_env_path = "${params.cogent_root}" + '/config/env/dna_report.yaml'

process DNA_REPORT {
    publishDir "${params.output_dir}", mode: 'copy'
    conda conda_env_path 

    input:
        path (ginkgo_ref)
        path (data)
        path (seg_copy)
        val (bin_size_actual)
        val (read_length_actual)


    output:
        path "dna_report/CogentDS_scDNA_analysis.rds", emit: cogentds_dna_object
        path "dna_report/CogentDS_scDNA_analysis_report.html", emit: html_report

    script:
        def bin_file = "variable_${bin_size_actual}_${read_length_actual}_bowtie"

        """
        export R_LIBS="./lib"
        
        Rscript ${cogentds_script} --pathtoCogentDSreporter ${cogentds_reporter} \
        --gc_file ${ginkgo_ref}/GC_${bin_file} --ginkgo_file ${data} \
        --ccn_file ${seg_copy}  \
        --bounds_file ${ginkgo_ref}/bounds_${bin_file} \
        --outPath \$PWD/dna_report --outCCNObject CogentDS_scDNA_analysis.rds \
        --outHTMLReport CogentDS_scDNA_analysis_report.html
        """
}
